#include "GridPeriodic.h"

GridPeriodic::GridPeriodic(vect_f Box_Width, double Lattice_Const)
{
    this->lattice_const = Lattice_Const;
    this->box_width = Box_Width;

    int grid_x = (int) (Box_Width.xcomp()/lattice_const);
    int grid_y = (int) (Box_Width.ycomp()/lattice_const);
    int grid_z = (int) (Box_Width.zcomp()/lattice_const);
    num_boxes = grid_x*grid_y*grid_z;

    this->grid_num = Vect3d<int>(grid_x,grid_y,grid_z);
    this->box = new vector<int>[grid_x*grid_y*grid_z];
    
    monomer_list = new vector<int>();
    inv_box_width.set(1/Box_Width.xcomp(),1/Box_Width.ycomp(),1/Box_Width.zcomp());
    rem_count = 0;
    Stop=0;
}

void GridPeriodic::freeGrid() {
    delete [] box;
    delete &grid_num;
    delete this;
    }


void printMembers (vector <int>  * bp, vect_i  * i_box, int * flag) {
       for(int i=0; i < bp->size();i++) 
       {
	  cout << i_box[0] << (*bp)[i] << "\n" << endl;
       }
}

void GridPeriodic::iterateWholeGrid(void (*func)(vector<int> * BoxPointer, vect_i * IntBox, int * flag)) {
   vect_i i_min;
   vect_i i_max;
   i_max.set(grid_num.xcomp()-1,grid_num.ycomp()-1,grid_num.zcomp()-1);
   BoxIterator * box_iter = new BoxIterator(i_min,i_max);
   do
   {
       vect_i * i_box = box_iter->getVect3d();
       int flg=0;
       int indx = grid_index(i_box[0]);
       vector<int> * bp = box + indx; 
       func(bp,i_box, &flg);
       delete i_box;

   } while(box_iter->next());
   delete box_iter;
}

void GridPeriodic::printAll() {
   iterateWholeGrid(printMembers);
}

void isBoxEmpty(vector <int>  * bp, vect_i  * i_box, int * flag) {

   if (bp->size() != 0) {
      int flg=0;
      cout << "Should be empty but isn't at:\n" << endl;
      printMembers(bp,i_box,&flg);
   }
}


void GridPeriodic::isGridEmpty() {
   iterateWholeGrid(isBoxEmpty);
}

/*
void GridPeriodic::printAll() {
   vect_i i_min;
   vect_i i_max;
   i_max.set(grid_num.xcomp()-1,grid_num.ycomp()-1,grid_num.zcomp()-1);
   BoxIterator * box_iter = new BoxIterator(i_min,i_max);
   do
   {
       vect_i * i_box = box_iter->getVect3d();
       int indx = grid_index(i_box[0]);
       vector<int> * bp = box + indx; 
       for(int i=0; i < bp->size();i++) 
       {
	  cout << i_box[0] << (*bp)[i] << "\n" << endl;
       }

   } while(box_iter->next());
}
*/

#if 0
int main() {
   int n=7;
   vect_f bsize; 
   bsize.set(10.0,10.0,10.0);
   GridPeriodic * grid = new GridPeriodic(bsize,1.0);
   vect_i box_i;
   for(int j = 0; j < 2; j++)
   {
      for(int i = 0; i < n; i++)
      {
	 vect_f r;
	 r.set((i+0.5),(i+0.5),(i+0.5));
	 //r.set((0.5),(0.5),(0.5));
	 grid->get_box(r, box_i);
	 grid->put(box_i,i);
      }
   grid->printAll();
      for(int i = 0; i < n; i++)
      {
	 vect_f r;
	 r.set(i+0.5,i+0.5,i+0.5);
	 //r.set(0.5,0.5,0.5);
	 grid->get_box(r, box_i);
	 grid->erase(box_i,i);
      }
   }

   grid->printAll();
   return 1;
}
#endif

